/* ====================================
 * KrakenSDR DOA Antenna Array Template
 * ====================================
 * Version 21 January 2023
 */

r_min			= 70; 						// Minimum array radius (1'800 MHz)
d_ant			= 44;						// Diameter of the antenna foot
x_arm			= r_min-d_ant/2+3*d_ant;	// Length of the arm
y_arm			= r_min-d_ant/2;			// Width of the arm and the extension
z_ant			= 6;						// z-axis print thickness

x_extension		= 4*d_ant;					// Length of the extension

d_pentagon		= 105;						// Diameter of the hub
d_magnet		= 10;						// Diameter of the magnets

min_antenna		= 75;						// Min. electrical antenna length
socket			= 130-min_antenna;			// Top of magnet to electrical antenna foot
mhz_scale		= [1700,1600,1400,1250,1150,1100,1001,1000,900,800,700,600,500,450,433,400,300,350];
fontsize_length	= 5;

draft			= 0.002;					// To make OpenSCAD's draft mode look a bit nicer
$fn				= 180;

/* =======
 * Objects
 * =======
 */
// Comment/uncomment as required:
	printall();								// For printing
//	show();									// For viewing only
//	show_printbed();						// For viewing only

/* =======
 * Modules
 * =======
 */
module printall() { // 200 mm x 200 mm printbed arrangement
	hub();
	translate([-60,-35,0]) arm();
	translate([-48,-85,0]) extension();
}

module show() {
	hub();
	arm();
	translate([x_arm,0,0]) extension();
	
// Show antennas and magnets:
//	antennapos_1700(r_min+0*d_ant,d_ant); // 1'700 MHz position
//	antennapos_1700(r_min+1*d_ant,d_ant); // 1'100 MHz position
	antennapos_1700(r_min+2*d_ant,d_ant); //   800 MHz Position
//	antennapos_1700(r_min+6*d_ant,d_ant); ((   600 MHz Position
}

module show_printbed() { // 200 mm x 200 mm printbed arrangement
	hub();
	translate([-60,-35,0]) arm();
	translate([-48,-85,0]) extension();

	translate([41,-45,0]) printbed();
	translate([35,-168,0]) linear_extrude(height=5) text("200 mm",halign="center");
	translate([160,-50,0]) rotate([0,0,90]) linear_extrude(height=5) text("200 mm",halign="center");
}

module hub() {
	difference() {
		minkowski(convexity=4) {
			dmink = 5;
			difference()  {
				// Pentagon
				cylinder(d=d_pentagon-dmink,h=z_ant,$fn=5);
				// 5 cam cutouts
				for (i = [0:360/5:359]) {
					rotate([0,0,i]) cam(1+dmink,0.4+dmink,3*draft);
				}
			}
			cylinder(d=dmink,h=0.001,$fn=16);
		}
		
		translate([0,0,-draft]) {
			// 5 magnet cutouts
			for (i = [0:360/5:359]) {
				rotate([0,0,i]) translate([d_pentagon/2-15,0,0]) {
					cylinder(d=d_magnet/2,h=z_ant+3*draft);
					translate([0,0,0.3]) cylinder(d=d_magnet+0.3,h=z_ant+3*draft);
				}
			}
			// 5 antenna cutouts for innermost position
			antennapos_1700(r_min,d_ant+1);
		}
	}
}

module arm() {
	difference() {
		union() {
			// Arm
			translate([x_arm/2,-y_arm/2,z_ant/2]) cube([x_arm,y_arm,z_ant],true);
			// MHz and antenna length meassured from top of base
			frequency("Max MHz",4,	47+0*d_ant);
			frequency("1800",	6,	70+0*d_ant);
			frequency("1100",	6,	70+1*d_ant);
			frequency("800",	6,	70+2*d_ant);
			// Antenna length scale
			color("GRAY") translate([52,-35,0]) 
				linear_extrude(height=z_ant+1,convexity=4) text("Ant. Length",size=4,halign="center",$fn=32);
			for (mhz = mhz_scale) {
				antennalength(mhz,x_extension,x_arm);
			}
		}
		
		translate([0,0,-draft]) {
			// Hub interface cutout
			cylinder(d=d_pentagon+2,h=z_ant+1+3*draft,$fn=5);
			rotate([0,0,20]) translate([0,-r_min+10,0]) cylinder(d=d_ant,h=z_ant+3*draft);
			rotate([0,0,20]) translate([-12,-r_min+25,0]) cylinder(d=20,h=z_ant+3*draft);
			// Antenna cutouts along the arm
			for (i = [0:d_ant:x_arm-r_min]) {
				translate([r_min-d_ant/2+d_ant/2+i,0,0]) {
					cylinder(d=d_ant,h=z_ant+3*draft);
					translate([0,0,z_ant/2]) cube([d_ant,8,z_ant+3*draft],true);
				}
			}
		}
	}
	// Cam into hub
	minkowski(convexity=4) {
		dmink = 5;
		cam(-dmink,-dmink,0);
		cylinder(d=dmink,h=0.001,$fn=16);
	}
}

module extension() {
	difference() {
		union() {
			// Extensions body
			translate([x_extension/2,-y_arm/2,z_ant/2]) cube([x_extension,y_arm,z_ant],true);
			// MHz and antenna length meassured from top of base
			frequency("600",	6,	d_ant/2+0*d_ant);
			frequency("500",	6,	d_ant/2+1*d_ant);
			frequency("450",	6,	d_ant/2+2*d_ant);
			frequency("400",	6,	d_ant/2+3*d_ant);
			// Antenna length scale
			for (mhz = mhz_scale) {
				antennalength(mhz,0,x_extension);
			}
		}
		
		// Antenna cutouts along the arm
		for (i = [d_ant/2:d_ant:x_extension-d_ant/2]) {
			translate([i,0,-draft]) {
				cylinder(d=d_ant,h=z_ant+3*draft);
				translate([0,0,z_ant/2]) cube([d_ant,8,z_ant+3*draft],true);
			}
		}
	}
}

module frequency(MHz,fontsize,x_offset) {
	color("GRAY") translate([x_offset,-25,0]) 
		linear_extrude(height=z_ant+1,convexity=4) 
			text(MHz,size=fontsize,halign="center",valign="center",$fn=32);
}

module antennalength(mhz,x_offset,x_length) {
	wavelength = 300000/mhz;
	el_antenna_length = (wavelength/4 >= min_antenna) ? wavelength/4 : (wavelength/4+wavelength/2) ;
	phy_antenna_length = el_antenna_length+socket;
	
	if ((phy_antenna_length < (x_offset+x_length)) && (phy_antenna_length > x_offset)) {
		color("GRAY") 
		if (wavelength/4 >= min_antenna) {
			translate([x_length+x_offset-phy_antenna_length,-y_arm,z_ant]) rotate([0,0,90]) 
				linear_extrude(height=1,convexity=4) 
					text(str(mhz),size=fontsize_length,halign="left",valign="center",$fn=32);
		} else {
			translate([x_length+x_offset-phy_antenna_length,-y_arm+5,z_ant]) rotate([0,0,90]) 
				linear_extrude(height=1,convexity=4) 
					text(str(mhz),size=fontsize_length,halign="left",valign="center",$fn=32);
		}
	}
}

module cam(delta_x,delta_y,delta_z) {
	rotate([0,0,0-360/5/2]) translate([25+20/2+8,4,z_ant/2]) rotate([0,0,-360/5/2]) 
		cube([26+delta_x,10+delta_y,z_ant+delta_z],true);
}

module antennapos_1700(r_min,d_ant) {
	color("GRAY") for (i = [0:360/5:359]) {
		rotate([0,0,i]) translate([r_min,0,0]) {
			cylinder(d=d_ant,h=z_ant+3*draft);
			cylinder(d=6,h=120);
		}
		rotate([0,0,i]) translate([d_pentagon/2-15,0,0.3]) cylinder(d=d_magnet,h=5);
	}
}

module printbed() {
	difference() {
		cube([200,200,1],true);
		translate([0,0,-draft]) cube([200-1,200-1,2],true);
	}
}

module minradius() {
	difference() {
		cylinder(r=r_min,h=1);
		translate([0,0,-2*draft]) cylinder(r=r_min-1,h=2);
	}
}
